--
-- ProfiLua
--
-- MIDI Example: Acquire Akai MPK49 potentiometer and slider values.
--
-- ***********************************
-- * REQUIRES AT LEAST PROFILUA_V09c *
-- ***********************************
--
-- (c) ASkr, www.askrprojects.net
--


gNumInputs  = 0
gNumOutputs = 8

InputNames  = {}
OutputNames = {"K1","K2","K3","K4","F1","F2","F3","F4"}


function gInputName(channel)
	return InputNames[channel]
end

function gOutputName(channel)
	return OutputNames[channel]
end


-- The Akai does not come with a custom driver, hence its name
-- is determined by Windows. On a German system, e.g.: "USB Audiogert".
-- Not that this won't work if you have other devices with "USB"
-- in their names, and the Akai is not the first one...
-- If you have same names multiple times, e.g.:
--   "USB audio device"
--   "USB audio device [2]"
--   "USB audio device [3]"
--   "USB audio device [4]"
-- You can change the lookup string value to, e.g.: "device [3]".
devInName = "USB"
devIN     = -1


------------------------------------------------------------------------------
-- MidiFindByName
--
-- Returns the MIDI device number of the first MIDIIN or MIDIOUT (dir)
-- device (0..xxx) that contains the string <name>.
--
-- E.g.: If looking for "Akai Blabla Sequencer", you may call this with
-- "Akai", "Akai Bla" or even "bla Seq".
------------------------------------------------------------------------------
function MidiFindFirstByName(dir, name)
	local i
	
	if dir == MIDIIN then
		for i=0, promidi.CountDevices(MIDIIN)-1 do
			if string.match(promidi.GetMidiInName(i),name) ~= nil then
				return i
			end
		end

	elseif dir == MIDIOUT then
		for i=0, promidi.CountDevices(MIDIOUT)-1 do
			if string.match(promidi.GetMidiOutName(i),name) ~= nil then
				return i
			end
		end
	end

	return -1
	
end



------------------------------------------------------------------------------
-- MidiInit
--
-- 
------------------------------------------------------------------------------
function MidiInit()
	local i,j

	print("\nMIDI IN devices:")
	j = promidi.CountDevices(MIDIIN)
	if j > 0 then
		for i=0,j-1 do
			print(string.format(" %3d -> ",i),promidi.GetMidiInName(i))
		end
	else
		print(" none")
	end

	i = MidiFindFirstByName(MIDIIN, devInName)

	if i >= 0 then
		if promidi.OpenMidiIn(i) == 0 then
			devIN = i
		end
	end
	
	if devIN < 0 then
		print("unable to open device: "..devInName)
	else
		print("successfully opened device: "..devInName)
	end
end



------------------------------------------------------------------------------
-- gSimStart
--
-- 
------------------------------------------------------------------------------
function gSimStart(In)

	MidiInit()
	return {0,0,0,0,0,0,0,0}
	
end



------------------------------------------------------------------------------
-- gSimStop
--
-- 
------------------------------------------------------------------------------
function gSimStop(In)
	
	if devIN >= 0 then
		promidi.CloseMidiIn(devIN)
	end
	
end


AkaiStates = {0,0,0,0,0,0,0,0}
------------------------------------------------------------------------------
-- gCalculate
--
-- 
------------------------------------------------------------------------------
function gCalculate(In)
	local ev
	
	ev = promidi.GetMessage(devIN)
	if ev ~= nil then
		if ev[1] == CTRL and ev[2] == 1 then
			if ev[3] == 0x16 then AkaiStates[1] = ev[4] end
			if ev[3] == 0x17 then AkaiStates[2] = ev[4] end
			if ev[3] == 0x18 then AkaiStates[3] = ev[4] end
			if ev[3] == 0x19 then AkaiStates[4] = ev[4] end
			if ev[3] == 0x0c then AkaiStates[5] = ev[4] end
			if ev[3] == 0x0d then AkaiStates[6] = ev[4] end
			if ev[3] == 0x0e then AkaiStates[7] = ev[4] end
			if ev[3] == 0x0f then AkaiStates[8] = ev[4] end
		end
	end

	-- Notice that "AkaiStates" already is a table, hence it
	-- does not require {} around it...
	return AkaiStates
	
end
